require('MainActor')
require('Level')
require('OrbitCamera')

BoxPlatform = {}

KeyMap = {
	runForward = 17, -- W,
	runBackward = 31, -- S
	runLeft = 30, -- A
	runRight = 32, -- D
	viewUp = 200, -- Up
	viewDown = 208, -- Down
	viewLeft = 203, -- Left
	viewRight = 205, -- Right
	jump = 57 -- Space
}

function BoxPlatform:New(o)
	o = o or {}   -- create object if user does not provide one
	setmetatable(o, self)
	self.__index = self

	self.level = Level:New()
	self.mainActor = MainActor:New()
	self.orbitCamera = OrbitCamera:New()
	self.orbitCamera.lookAtEntity = self.mainActor.body

	return o
end

function BoxPlatform:OnBeforeUpdate()
	self:ProcessInput()
	self.level:OnBeforeUpdate()
	self.mainActor:OnBeforeUpdate()
end

function BoxPlatform:OnAfterUpdate()
	self.orbitCamera:OnAfterUpdate()
end

function BoxPlatform:ProcessInput()
	Input.ControllerManager_Instance():Update()

	local direction = Math.Vector3(0, 0, 0)
	local moveSped = 0
	local viewYaw = 0
	local viewPitch = 0
	local jump
	
	local xbox360Controller = Input.ControllerManager_Instance():GetController(2)
	
	if xbox360Controller:IsConnected() then
		local thumbLeftX = xbox360Controller:GetCurrentState(16)
		local thumbLeftY = xbox360Controller:GetCurrentState(17)
		
		if thumbLeftX <= -0.5 or thumbLeftX > 0.5 then
			direction.x = thumbLeftX
		end
		
		if thumbLeftY <= -0.5 or thumbLeftY > 0.5 then
			direction.z = -thumbLeftY
		end
		
		moveSpeed = direction:GetLength()
		
		jump = xbox360Controller:GetCurrentState(10) == 1 and xbox360Controller:GetLastState(10) == 0

		local thumbRightX = xbox360Controller:GetCurrentState(18)
		local thumbRightY = xbox360Controller:GetCurrentState(19)
		
		if thumbRightX < -0.25 or thumbRightX > 0.25 then
			viewYaw = thumbRightX
		end
		
		if thumbRightY < -0.25 or thumbRightY > 0.25 then
			viewPitch = -thumbRightY
		end		
	else
		local keyboard = Input.ControllerManager_Instance():GetController(0)

		direction.z = -keyboard:GetCurrentState(KeyMap.runForward) + keyboard:GetCurrentState(KeyMap.runBackward)
		direction.x = -keyboard:GetCurrentState(KeyMap.runLeft) + keyboard:GetCurrentState(KeyMap.runRight)
		moveSpeed = 1
		
		viewYaw = -keyboard:GetCurrentState(KeyMap.viewLeft) + keyboard:GetCurrentState(KeyMap.viewRight)
		viewPitch = -keyboard:GetCurrentState(KeyMap.viewUp) + keyboard:GetCurrentState(KeyMap.viewDown)
		
		jump = keyboard:GetCurrentState(KeyMap.jump) == 1 and keyboard:GetLastState(KeyMap.jump) == 0

		end
	
	if direction.x ~= 0 or direction.z ~= 0 then
		local viewDirection = self.orbitCamera.camera:GetWorldTransform():TransformVector(direction)
		viewDirection.y = 0		
		self.mainActor:Run(viewDirection:GetNormalized() * moveSpeed)
	else
		self.mainActor:Stop()
	end
	
	if jump then
		self.mainActor:Jump()
	end
	
	-- Rotate camera also
	local runRotateSpeed = 1.0
	local yawRotateSpeed = 2.0
	local pitchRotateSpeed = 2.0
	local timeStep = Engine.Game_Instance():GetTimeStep()
	
	self.orbitCamera.yaw = self.orbitCamera.yaw + (direction.x * runRotateSpeed + viewYaw * yawRotateSpeed) * timeStep
	self.orbitCamera.pitch = self.orbitCamera.pitch + viewPitch * pitchRotateSpeed * timeStep
	if self.orbitCamera.pitch > 1.5 then
		self.orbitCamera.pitch = 1.5
	elseif self.orbitCamera.pitch < 0 then
		self.orbitCamera.pitch = 0
	end
end

--
local test

function OnBeforeUpdate()
	test = test or BoxPlatform:New()
	test:OnBeforeUpdate()
end

function OnAfterUpdate()
	test:OnAfterUpdate()
end
