MainActor = {}

function lerp(a, b, t)
	return a * (1 - t) + b * t
end

function MainActor:New(o)
	o = o or {}   -- create object if user does not provide one
	setmetatable(o, self)
	self.__index = self
	
	self.orientation = 0
	self.groundVelocity = Math.Vector3(0, 0, 0)

	self.material = Graphics.Material("Effect/blinn")
	self.material:SetTexture(
		self.material:GetParameterByName("ColorTexture"), 
		Graphics.Texture("Collada/Seymour/boy_10.tga"))
	
	local world = Engine.Game_Instance():GetWorld()
	
	-- body
	self.body = world:CreateEntity()
	self.body:SetName("mainActorBody")
	self.body:SetLocalTransform(Math.Matrix44_Translate(Math.Vector3(0, 5, 0)))

	-- body physcial
	self.bodyPhysical = Engine.Physical_Cast(self.body:CreateComponent(Engine.PhysicalComponentType))
	local rigidBody = self.bodyPhysical:GetRigidBody()
	--rigidBody:SetMass(50) -- still not working after applying bullet's patch (http://code.google.com/p/bullet/source/detail?r=1561)
	rigidBody:DisableRotation()
	rigidBody:SetAngularSleepingThreshold(0)
	rigidBody:SetLinearSleepingThreshold(0)
	
	self.bodyCollider = Physics.CapsuleCollider()
	rigidBody:SetCollider(self.bodyCollider)

	-- skin	
	self.skin = world:CreateEntity()
	self.skin:SetName("skin")
	local pi = 3.14159
	self.skin:SetLocalTransform(
		Math.Matrix44_Translate(Math.Vector3(0, 0, -2)) * 
		Math.Matrix44_RotateAxis(Math.Vector3(1, 0, 0), pi * 0.5) *
		Math.Matrix44_RotateAxis(Math.Vector3(0, 1, 0), pi) *
		Math.Matrix44_Scale(Math.Vector3(0.5, 0.5, 0.5)))
	self.skin:Link(self.body)
	
	local builder = Graphics.GeometryBuilder()
	self.skinGeometry = Graphics.Geometry()
	builder:Load("Collada/Seymour/Seymour.geo")
	builder:Build(self.skinGeometry)
	--Graphics.ChamferBox_Create(self.skinGeometry, Math.Vector3(0.5, 0.5, 0.5), 0.1, 2)
	
	local mesh = Engine.Mesh_Cast(self.skin:CreateComponent(Engine.MeshComponentType))
	mesh:SetGeometry(self.skinGeometry)
	mesh:SetMaterial(self.material)
	
	return o
end

function MainActor:Run(direction)
	local timeStep = Engine.Game_Instance():GetTimeStep()

	-- move
	-- uncomment the following line to disable moving in the air
	--if self.groundTouch then
		local speed = 5
		local factor = 10
		local velocity = direction * speed
		local impulse = (self.groundVelocity + velocity - self.bodyPhysical:GetRigidBody():GetLinearVelocity()) * factor * timeStep
		impulse.y = 0
		self.bodyPhysical:GetRigidBody():ApplyImpulse(impulse, Math.Vector3(0, 0, 0))
	--end
	
	-- rotate
	targetOrientation = math.atan2(direction.x, -direction.z)

	local pi = 3.14159
	local rotateSpeed = 5
	
	if self.orientation - targetOrientation > pi then
		self.orientation = self.orientation - pi * 2
	end

	if self.orientation - targetOrientation < -pi then
		self.orientation = self.orientation + pi * 2
	end

	self.orientation = lerp(self.orientation, targetOrientation, math.min(timeStep * rotateSpeed, 1))

	local t = self.bodyPhysical:GetRigidBody():GetWorldTransform()
	local r = Math.Matrix44_RotateAxis(Math.Vector3(0, 1, 0), self.orientation)
	r.m03 = t.m03
	r.m13 = t.m13
	r.m23 = t.m23
	self.bodyPhysical:GetRigidBody():SetWorldTransform(r)
end

function MainActor:Stop()
	if not self.groundTouch then return end

	local damping = 20 * Engine.Game_Instance():GetTimeStep()
	local impulse = (self.groundVelocity - self.bodyPhysical:GetRigidBody():GetLinearVelocity()) * damping
	impulse.y = 0
	self.bodyPhysical:GetRigidBody():ApplyImpulse(impulse, Math.Vector3(0, 0, 0))	
end

function MainActor:Jump()
	if not self.groundTouch then return end
	
	local impulse = Math.Vector3(0, 6, 0)
	self.bodyPhysical:GetRigidBody():ApplyImpulse(impulse, Math.Vector3(0, 0, 0))
end

function MainActor:OnBeforeUpdate()
	local t = self.body:GetWorldTransform()
	local from = Math.Vector3(t.m03, t.m13, t.m23) 
	local to = from + Math.Vector3(0, -1.2, 0)
	local results = Physics.RayTestResultList()      
	
	if Engine.Game_Instance():GetDynamicsWorld():RayTest(from, to, results) then
		self.groundTouch = true
		self.groundNormal = results[0].normal
		self.groundVelocity = results[0].rigidBody:GetLinearVelocity()
	else
		self.groundTouch = false
	end
end
